@@ -20,8 +20,15 @@ ecodat_cate[['gdp']] = setDT(list(
 # main economic data by category
 # 
 # export
-ed_cate = function(cate=NULL, symbol=NULL, date_range = "3y", from = NULL, to = Sys.Date(), print_step = 1L, ...) {
+ed_cate_current = function(area='top') {
+    area = check_arg(area, c('world', 'europe', 'america', 'asia', 'africa', 'australia', 'top'))
     
+    wb = read_html(sprintf('https://tradingeconomics.com/matrix?g=%s', area)) 
+    tbl = setDT(html_table(wb, fill = TRUE)[[1]])
+    setnames(tbl, gsub('[ \\./]+', '_', tolower(names(tbl))))
+    setnames(tbl, '', 'area')
+    
+    return(tbl)
 }
 # BIZ
 # GDP	GDP YoY	GDP QoQ	
